package com.perforce.hws.core.p4base;

import com.perforce.p4java.exception.MessageSeverityCode;
import com.perforce.p4java.impl.mapbased.rpc.msg.RpcMessage;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * Helper methods around the main data returned from the p4d server.
 * <br/>
 * Almost all data is represented as an array of Map&lt;String, Object&gt;.
 * We <em>frequently</em> need convert result data that gets returned in
 * different formats.
 */
public class ResultMap implements Map<String, Object> {

    // Internal map object that holds data returned from the server.
    private Map<String, Object> internal;

    public ResultMap(Map<String, Object> internal) {
        this.internal = internal;
    }

    /**
     * Convert the parameter using common internal conversion mechanisms.
     *
     * @param key
     * @param clazz
     * @param <T>
     * @return
     * @throws IllegalArgumentException When the value is not null and we can
     *                                  not convert it the intended data format.
     */
    public <T> T get(Object key, Class<T> clazz) {
        Object value = get(key);
        if (value == null) {
            return null;
        }
//        if (canConvert(key, value, clazz)) {
//            return convert(key, value, clazz);
//        }
        if (clazz.isInstance(value)) {
            return (T) value;
        }
        String msg = String.format("%s is not convertable to %s", value, clazz);
        throw new IllegalArgumentException(msg);
    }

    /**
     * Retuorns the severity level if set in a key that looks like "code0".
     *
     * @return One of the constants in MessageSeverityCode, defaults to E_EMPTY
     */
    public int getMaxSeverity() {
        return entrySet().stream()
                .filter((e) -> e.getKey().startsWith("code"))
                .map((e) -> RpcMessage.getSeverity((String)e.getValue()))
                .max(Integer::compare)
                .orElse(MessageSeverityCode.E_EMPTY);
    }


    public List<ErrorId> getErrors() {
        return entrySet().stream()
                .filter((e) -> e.getKey().startsWith("code"))
                .map((e) -> new ErrorId((String)e.getValue()))
                .collect(Collectors.toList());
    }

    /**
     * Return the entries that start with the particular prefix, in order.
     *
     * @param prefix Typically something like "View"
     * @return The list of each key/value pair that starts with the prefix.
     */
    public List<Entry<String, Object>> indexedEntries(String prefix) {
        return entrySet().stream()
                .filter((e) -> e.getKey().startsWith(prefix))
                .sorted((x,y) -> {
                    int n1 = Integer.valueOf(x.getKey().substring(prefix.length()));
                    int n2 = Integer.valueOf(y.getKey().substring(prefix.length()));
                    return Integer.compare(n1, n2);
                })
                .collect(Collectors.toList());
    }

    @Override
    public String toString() {
        StringBuilder sb = new StringBuilder();
        sb.append("{");
        sb.append(entrySet().stream()
                .map((e) -> {
                    StringBuilder s = new StringBuilder();
                    s.append('"');
                    s.append(e.getKey());
                    s.append('"');
                    s.append(": ");
                    s.append('"');
                    s.append(String.format("%s", e.getValue()));
                    s.append('"');
                    return s.toString();
                })
                .collect(Collectors.joining(", ")));
        sb.append("}");
        return sb.toString();
    }

    //-----------------------------------------------------------------------
    // Methods that just delegate to the underlying map
    //-----------------------------------------------------------------------

    @Override
    public int size() {
        return internal.size();
    }

    @Override
    public boolean isEmpty() {
        return internal.isEmpty();
    }

    @Override
    public boolean containsKey(Object key) {
        return internal.containsKey(key);
    }

    @Override
    public boolean containsValue(Object value) {
        return internal.containsValue(value);
    }

    @Override
    public Object get(Object key) {
        return internal.get(key);
    }

    @Override
    public Object put(String key, Object value) {
        return internal.put(key, value);
    }

    @Override
    public Object remove(Object key) {
        return internal.remove(key);
    }

    @Override
    public void putAll(Map<? extends String, ?> m) {
        internal.putAll(m);
    }

    @Override
    public void clear() {
        internal.clear();
    }

    @Override
    public Set<String> keySet() {
        return internal.keySet();
    }

    @Override
    public Collection<Object> values() {
        return internal.values();
    }

    @Override
    public Set<Entry<String, Object>> entrySet() {
        return internal.entrySet();
    }
}
